<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;

use Teknisa\MAN\Services\RelatorioService;
use Teknisa\MAN\Utilities\Resposta;
use Teknisa\MAN\Utilities;

use Zeedhi\Framework\Controller\CrudWithParameterBag;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ManutencaoVendaDashboard extends CrudWithParameterBag {
    
	protected $dataSourceName = 'manutencaodavendadashboard';
	protected $entityManager;
    protected $parameterBagColumns = ["CDFILIAL", "TCDFILIAL", "DTINICIO", "DTFINAL"];
    
    protected $relatorioService;
    
    public static $logMessages = array();
	
	public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, RelatorioService $relatorioService, \Doctrine\ORM\EntityManager $entityManager) {
    	$this->relatorioService = $relatorioService;
    	$this->entityManager = $entityManager;
        parent::__construct($dataSourceManager, $parameterBag);
    }
    
    public static function escreverLog ($data) {
        array_push(self::$logMessages, $data);
    }
    
    public static function getLog () {
        return self::$logMessages;
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        
        $conditions = $this->relatorioService->getParamsAsArray($request);        // Some params
		$conditions = $this->relatorioService->addParams($conditions, 'CDFILIAL');
		$this->relatorioService->setIntervalAsDTINIandDTFIM($conditions, 'DTENTRVENDA', 'DTINICIO', 'DTFINAL');
		
		// Gets conditions that use the operator LIKE
		$likeableConditions = $this->relatorioService->getLikeConditions($request);
		
		// Removes all Likeable conditions from the conditions array
		// So we can use relatorioServices's createNewZeedhiFilterRequestObject's method
		foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
		}
		
		// Builds a new Zeedhi's FilterCriteria's Request's object, with the remaining conditions
		$request = $this->relatorioService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions); 
		
		// Adds to this new Request, all the likeable conditions
		foreach ($likeableConditions as $condition) {
			$request->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
        
        parent::find($request, $response);
    }
    
    public function getTipoDashboard(Request\Row $request, DTO\Response $response) {
        
        try{
            
            $query = "SELECT IDDASHBOARD, 
                    (CASE 
                        WHEN IDDASHBOARD = 'C' THEN 'Corporativo' 
                        WHEN IDDASHBOARD = 'P' THEN 'Padrão'
                        ELSE 'NAO ATRIBUÍDO'
                    END) AS NMDASHBOARD
                    FROM PARAMGERAL";   
                
           $resultado = $this->entityManager->getConnection()->fetchAssoc($query);
            
        }catch(\Exception $e){
            
        }
        
        $response->addDataSet(new DataSet('resultado', $resultado));
    }
    
    public function findLog(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'manutencaodavendadashboardLog';
        parent::find($request, $response);
    }
    
    public function saveLog(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        if(!empty($rows)){
            $row = $rows[0];
            $data = new \DateTime(str_replace("/","-",$row['DTENTRVENDA']['start']));
            $dataFinal = new \DateTime(str_replace("/","-",$row['DTENTRVENDA']['end']));
            while($data <= $dataFinal){
                $filiais = $row['CDFILIAL'];
                foreach($filiais as $value){
                    $params[':CDFILIAL'] = $value;
                    $params[':DTLOGDASH'] = $data->format('d/m/Y');
                    $params[':DTINCLUSAO'] = date('d/m/Y');
                    $params[':CDOPERINCLUSAO'] =  $row['P_CDOPERADOR'];
                    
                    $this->insertLog($params);   
                }
                $data = $data->add(new \DateInterval('P1D'));
            }
        }
    }
    
    public function vendasDashboard(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $retorno = new Resposta();
        
        try{
            
            if($row['CONTROLE'] == 1){
                $this->getLogOperacao(1, 'Recalculando', '', $row['CDOPERADORLOCAL']);
            }
            
            
            self::escreverLog("Atualizando informações de: ".$row['LABELUNIDADE']. " - Data: " . $row['DTVENDA']);
            
            //chamar a funcao que ira executar a procedure
            $result = $this->callProcedure($row);
            if($result['error']) {
                $this->getLogOperacao(2, $result['message'], 'E');
            } 
            
            if($row['CONTROLE'] == $row['CONTADOR']){
                $this->getLogOperacao(3, null, 'S');
            }
            
            $retorno->setStatus(Resposta::SUCESSO);
            $retorno->setMessage(
                array(
                    'log' => $this->getLog()
                )    
            );
            
        }catch(\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
    }
    
    public function deleteVendasDashboardLog(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $retorno = new Resposta();
        
        try{
            
            if(!empty($row)){
                foreach($row as $value){
                    $params[':CDFILIAL'] = $value['CODIGOUNID'];
                    $params[':DTLOGDASH'] = $value['DTVENDA'];
                    
                    //chamar a funcao que ira executar a procedure
                    $this->deleteVendaLog($params);       
                }
                
            }
            $retorno->setStatus(Resposta::SUCESSO);
            $retorno->setMessage(
                array(
                    'log' => $this->getLog()
                )    
            );
            
        }catch(\Exception $e){
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.'
                )
            );
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
    }
    
    public function vendasDashboardLog(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $retorno = new Resposta();
        
        try{
            $this->getLogOperacao(1, 'Recalculando', '', $row['CDOPERADORLOCAL']);
            
            self::escreverLog("Atualizando informações");
            
            //chamar a funcao que ira executar a procedure
            $result = $this->callProcedure($row, true);
            if($result['error']) {
                $this->getLogOperacao(2, $result['message'], 'E');
            } 
            
            $this->getLogOperacao(3, null, 'S');
            
            $retorno->setStatus(Resposta::SUCESSO);
            $retorno->setMessage(
                array(
                    'log' => $this->getLog()
                )    
            );
            
        }catch(\Exception $e){
            $this->getLogOperacao(3, null, 'E', $e->getMessage());
            $retorno->setStatus(Resposta::ERRO);
            $retorno->setError(true);
            $retorno->setMessage(
                array(
                    'text' => 'Erro! <br><br>'.$e->getMessage().'<br><br>Todas as alterações foram revertidas.',
                    'log'  =>  $this->getLog()
                )
            );
        }
        
        $response->addDataSet(new DataSet('resultado', $retorno->toPrint()));
    }
    
    private function callProcedure($data, $type = false){
    	
		$return = array(
    		'error' => false,
    		'message' => ''
    	);
    	
        $this->entityManager->beginTransaction();
        
        try{
            
            if($type){
              $query = "BEGIN	
                    DASHBOARD_VND_CARDS();	        
                END;";   
                $this->entityManager->getConnection()->executeQuery($query);
                
            }else{
            
               $params[':P_DTCALCULO1'] = $data['DTVENDA'];
               $params[':P_DTCALCULO2'] = $data['DTVENDA'];
               $params[':P_FILIAL'] = $data['CODIGOUNID'] ;
            
               $querySession = "ALTER SESSION SET NLS_DATE_FORMAT = 'DD/MM/YYYY'";
                
               $query = "BEGIN
                    	RECALCULA_DASHBOARD_PADRAO(TO_CHAR(TO_DATE(:P_DTCALCULO1,'DD/MM/YYYY'), 'DD/MM/YYYY'), TO_CHAR(TO_DATE(:P_DTCALCULO2), 'DD/MM/YYYY'), :P_FILIAL);
                     END;";
                     
               $this->entityManager->getConnection()->executeQuery($querySession);
               $this->entityManager->getConnection()->executeQuery($query, $params);
            }
                
            $this->entityManager->commit();
            
        }catch (\Exception $e){
        	$this->entityManager->rollback();
            $return['error'] = true;
            $return['message'] = $e->getMessage();
        }
        
        return $return;
    }
    
    private function insertLog($params){
        
        $return = array(
    		'error' => false,
    		'message' => ''
    	);
    	
        $this->entityManager->beginTransaction();
        
        try{
            
           $insert = "INSERT INTO LOGALTERADASH
						(CDFILIAL, DTLOGDASH, DTINCLUSAO, CDOPERINCLUSAO)
					VALUES
						(:CDFILIAL, :DTLOGDASH, :DTINCLUSAO, :CDOPERINCLUSAO)";
                 
         
           $this->entityManager->getConnection()->executeQuery($insert, $params);
        
            
        $this->entityManager->commit();
            
        }catch (\Exception $e){
        	$this->entityManager->rollback();
            $return['error'] = true;
            $return['message'] = $e->getMessage();
        }
        
        return $return;
    }
    
    private function deleteVendaLog($params){
        
        $return = array(
    		'error' => false,
    		'message' => ''
    	);
    	
        $this->entityManager->beginTransaction();
        
        try{
            
           $delete = "DELETE LOGALTERADASH
                        WHERE CDFILIAL = :CDFILIAL AND DTLOGDASH =  :DTLOGDASH
						";
         
           $this->entityManager->getConnection()->executeQuery($delete, $params);
        
            
        $this->entityManager->commit();
            
        }catch (\Exception $e){
        	$this->entityManager->rollback();
            $return['error'] = true;
            $return['message'] = $e->getMessage();
        }
        
        return $return;
    }
    
    public function getLogOperacao($type, $mensage = null, $operation = null, $params = array()){
        /* $type
         1. Inicio
         2. Meio
         3. Final
        */
        
        /* $operation
         S. Sucesso
         E. Exception
        */
        
        if($type == 1){
            self::escreverLog("Log " .$mensage. " ");
            self::escreverLog("Data da operação: ".date("d/m/Y H:i:s"));
            self::escreverLog("Operador: ".$params);
            self::escreverLog("");
        }
        
        if($type == 2 && $operation == 'E' && $mensage == null){
            self::escreverLog("Operação finalizada de maneira inesperada. Data: ".date("d/m/Y H:i:s"));
        }
        
        if($type == 2 && $operation == 'E' && !empty($mensage)){
            self::escreverLog(str_pad(' ERRO! ', strlen($mensage), "-", STR_PAD_BOTH));
            self::escreverLog("| ".$mensage." |");
            self::escreverLog(str_pad('-', strlen($mensage), "-", STR_PAD_LEFT));
            self::escreverLog("Operação finalizada de maneira inesperada. Data: ".date("d/m/Y H:i:s"));
        }
        
        if($type == 3 && $operation == 'S' && $mensage == null){
            self::escreverLog("");
            self::escreverLog("## Confirmando alterações ##");
            self::escreverLog("Operação finalizada com sucesso! Data: ".date("d/m/Y H:i:s"));
        }
        
        if($type == 3 && $operation == 'S' && !empty($mensage)){
            self::escreverLog("");
            self::escreverLog("## Confirmando alterações##");
        }
        
        if($type == 3 && $operation == 'E'){
            self::escreverLog("");
            self::escreverLog(str_pad(' ERRO! ', strlen($params), "-", STR_PAD_BOTH));
            self::escreverLog("| ".$params." |");
            self::escreverLog(str_pad('-', strlen($params), "-", STR_PAD_LEFT));
            self::escreverLog("");
            self::escreverLog("## Desfazendo alterações ##");
            self::escreverLog("Operação finalizada de maneira inesperada. Data: ".date("d/m/Y H:i:s"));
        }
        
    }
    
}